﻿using EDM.Complex.DAL;
using EDM.Complex.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EDM.Complex.BLL
{
    public class UserManager
    {
        public enum UserStatus
        {
            None,
            Denied,
            Failed,
            DontCare,
            Persisted
        }

        public UserServices Services { get; private set; }

        public UserManager(UserServices services)
        {
            Services = services;
        }

        public UserStatus CreateAndPersistUser(string userName,
            string email, int age, HashSet<string> interests)
        {

            // Input validation
            if (userName == null) throw new ArgumentNullException("userName cannot be null.");
            if (email == null) throw new ArgumentNullException("email cannot be null.");
            if (age <= 0) throw new ArgumentOutOfRangeException("age must be larger than zero.");
            if (interests == null) throw new ArgumentNullException("interests cannot be null.");

            // Age validation
            if (age < 18) return UserStatus.Denied;

            // Interests count validation
            if (interests.Count == 0) return UserStatus.DontCare;

            // Getting the Gravatar profile pic
            string profilePicUrl = "";
            if (Services != null) profilePicUrl =
                    Services.GetProfilePictureUrl(email);
            else throw new Exception("Services cannot be null.");

            // Persisting the user
            var result = DBExecutor.ExecuteInsert("storeUser",
                new object[] {
                    new User() { UserName = userName, Email = email,
                        Age = age, Interests = interests }
                });
            return result ? UserStatus.Persisted : UserStatus.Failed;
        }
    }
}
